@@ -39,9 +39,17 @@ def verify_wacz(wacz):
                 continue
 
         file = z.open(filename)
-        data = json.loads(file.read())
+        try:
+            data = json.loads(file.read())
+        except json.JSONDecodeError as e:
+            raise VerificationException(f"digest is ill-formed: {e}")
 
-        return verify(data["signedData"])
+        try:
+            return verify(data["signedData"])
+        except KeyError:
+            raise VerificationException(f"{wacz} is unsigned")
+
+    return VerificationException(f"{wacz} is missing datapackage-digest.json")
 
 
 def sign(string, dt):
@@ -125,7 +133,7 @@ def sign(string, dt):
     return {
         'software': f"wacz-signing {__version__}",
         'hash': string,
-        'created': dt,
+        'created': dt.strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
         'signature': signature,
         'timeSignature': time_signature,
         'domain': domain,
@@ -192,11 +200,11 @@ def verify(signed_req):
     created = ensure_dt(signed_req["created"])
 
     if cert.not_valid_before > created:
-        raise VerificationFailure(
+        raise VerificationException(
             "signature created before cert existence"
         )
     if created > cert.not_valid_before + duration:
-        raise VerificationFailure(
+        raise VerificationException(
             "signature created after claimed cert duration"
         )
 
@@ -216,9 +224,9 @@ def verify(signed_req):
     timestamp = rfc3161ng.get_timestamp(timestamp_token)
 
     if not timestamp:
-        raise VerificationFailure("unable to verify timestamp")
+        raise VerificationException("unable to verify timestamp")
 
-    check_range(created, timestamp, stamp_duration, VerificationFailure)
+    check_range(created, timestamp, stamp_duration, VerificationException)
 
     timestamp_certs = validate_cert_chain(
         ensure_bytes(signed_req["timestampCert"])
@@ -233,16 +241,21 @@ def verify(signed_req):
     return {
         'observer': ['mkcert'] if mkcert else domains,
         'software': signed_req['software'],
-        'timestamp': timestamp.strftime("%Y-%m-%dT%H:%M:%SZ")
+        'timestamp': timestamp.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
     }
 
 
 def ensure_bytes(cert):
-    return bytes(cert, encoding='ascii') if isinstance(cert, str) else cert
+    return bytes(cert, encoding="ascii") if isinstance(cert, str) else cert
 
 
 def ensure_dt(ts):
-    return ts if isinstance(ts, datetime) else datetime.strptime(ts, "%Y-%m-%dT%H:%M:%SZ")  # noqa
+    if isinstance(ts, datetime):
+        return ts
+    try:
+        return datetime.strptime(ts, "%Y-%m-%dT%H:%M:%S.%fZ")
+    except ValueError:
+        return datetime.strptime(ts, "%Y-%m-%dT%H:%M:%SZ")
 
 
 def get_fingerprint(cert):
@@ -318,7 +331,3 @@ class SigningException(BaseException):
 
 class VerificationException(BaseException):
     """ Raise for errors in verification """
-
-
-class VerificationFailure(BaseException):
-    """ Raise for errors in verification """
